#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

#include <U8g2lib.h>

#define backlight 12
//U8GLIB_PCD8544 u8g(14, 13, 2, 15, 0); 48856
U8G2_PCD8544_84X48_F_4W_SW_SPI u8g2(U8G2_R0, /* clock=*/ 14, /* data=*/ 13, /* cs=*/ 2, /* dc=*/ 15, /* reset=*/ 0);  // Nokia 5110 Display

const char* PROGMEM ssid = "ssid";
const char* PROGMEM password = "password";
int port = 23;

bool tg = 0;

char lines[8][22];
byte lline=7;
byte lineend=20;
byte llpos=255;

bool screenupdated=0;

unsigned long utimer = 0;

WiFiServer tserver(port);
WiFiClient tclient;

char zbuffer[32];

void setup() {
  Serial.begin(74880);
  pinMode(backlight, OUTPUT);
  digitalWrite(backlight, HIGH);
  u8g2.begin();
  
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_4x6_mf);
  u8g2.drawStr(0,5,PSTR("Connecting...."));
  u8g2.sendBuffer();
  
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while(WiFi.waitForConnectResult() != WL_CONNECTED) {
    delay(5000);
    ESP.restart();
  }
  ArduinoOTA.setPort(8266);
  ArduinoOTA.setHostname(PSTR("uScreen"));
  ArduinoOTA.onStart([](){
    u8g2.clearBuffer();
    u8g2.drawStr(0,5,PSTR("ArduinoOTA"));
    u8g2.sendBuffer();
    });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total){
    sprintf_P(zbuffer,PSTR("Progress: %u%%"), (progress / (total / 100)));
    u8g2.drawStr(0,11,zbuffer);
    u8g2.sendBuffer();
    });
  ArduinoOTA.onEnd([](){
    u8g2.drawStr(0,17,PSTR("Done."));
    u8g2.sendBuffer();
    });
  ArduinoOTA.begin();
  
  u8g2.clearBuffer();
  u8g2.drawStr(0,5,PSTR("Connected..."));
  
  sprintf_P(zbuffer,PSTR("IP:%d.%d.%d.%d"),WiFi.localIP()[0],WiFi.localIP()[1],WiFi.localIP()[2],WiFi.localIP()[3]);
  u8g2.drawStr(0,11,zbuffer);
  sprintf_P(zbuffer,PSTR("Heap:%d"),ESP.getFreeHeap());
  u8g2.drawStr(0,17,zbuffer);
  u8g2.sendBuffer();
  
  tserver.begin();
  delay(2000);
  for(byte i=0; i<=lline; i++){
    lines[i][lineend+1]='\0';     
  }
}

void handleTelnet(){
  if(tserver.hasClient()){ //client is connected
    if(!tclient||!tclient.connected()){ //no client available
      if(tclient){tclient.stop();}      //client disconnected
      tclient = tserver.available();    //ready for new client
    } else {
      tserver.available().stop();       //have client, block new conections
    }
  }
  if(tclient&&tclient.connected()&&tclient.available()){ //client input processing
    while(tclient.available()){
      utimer=millis()+20;
      char inc = tclient.read();
      bool printingchar=0;          
      if(inc!='\r'&&inc!='\n'){     //only add characters if they are printable
        printingchar=1;
        llpos++;                    //see what happens if character were added to end of last line
      }
      if(llpos>lineend||inc=='\n'){ //move the screen up a line if the last line is beyond full or carriage return
        llpos=0;
         for(byte i=0; i<=lline-1; i++){
          for(byte j=0; j<=lineend; j++){
            lines[i][j]=lines[i+1][j];
          }
          lines[i][lineend+1]='\0';
         }
         lines[lline][0] = '\0'; //end last line string at the beginning of the line
         if(!printingchar){llpos=255;} //back up so that next printing char will be put at start of line (0) 
      }
      if(printingchar){ //only add to the line if a valid character
        lines[lline][llpos] = inc;
        lines[lline][llpos+1] = '\0';
      }
      screenupdated=0;
    }
  } 
}

void loop(){
  ArduinoOTA.handle();
  handleTelnet();
  //tclient.println(millis());
  delay(20);
  if(utimer<millis()){
    if(screenupdated==0){
      screenupdated=1;
      u8g2.clearBuffer();
      for(byte i=0; i<=7; i++){
        u8g2.drawStr(0,5+i*6,lines[i]); 
      }
      u8g2.sendBuffer();
    }
  }
}
